@@ -1,13 +1,15 @@
+'''Query subclass used by Manager as default session query class.
+'''
 
 from math import ceil
 
 from sqlalchemy import orm, and_, or_, inspect
 from sqlalchemy.orm.strategy_options import Load
 
-from six.moves import reduce, map
-
 
 class Query(orm.Query):
+    '''Extension of default Query class used in SQLAlchemy session queries.
+    '''
 
     # when used as a query property (e.g. MyModel.query), then this is populated with the originating model
     __model__ = None
@@ -61,6 +63,7 @@ class Query(orm.Query):
         return self._join_eager(keys, True, **kargs)
 
     def _join_load(self, keys, load_type, **kargs):
+        '''Helper method for returning load strategies.'''
         opt = getattr(orm, load_type)(keys[0], **kargs)
 
         for k in keys[1:]:
@@ -96,29 +99,43 @@ class Query(orm.Query):
     def defer(self, *columns):
         '''Apply `defer()` to query'''
         obj, columns = get_load_options(*columns)
-        return self.options(reduce(lambda result, i: result.defer(i), columns, obj))
+        opts = obj
+        for column in columns:
+            opts = opts.defer(column)
+        return self.options(opts)
 
     def undefer(self, *columns):
         '''Apply `undefer()` to query'''
         obj, columns = get_load_options(*columns)
-        return self.options(reduce(lambda result, i: result.undefer(i), columns, obj))
+        opts = obj
+        for column in columns:
+            opts = opts.undefer(column)
+        return self.options(opts)
 
     def undefer_group(self, *names):
         '''Apply `undefer_group()` to query'''
         obj, names = get_load_options(*names)
         return self.options(obj.undefer_group(names[0]))
 
-    def map(self, function, *iterable_args):
+    def map(self, func):
         '''Call native `map` on `self.all()`'''
-        return list(map(function, self.all(), *iterable_args))
+        return [func(item) for item in self.all()]
 
-    def reduce(self, function, initial=None):
-        '''Call native `reduce` on `self.all()`'''
-        return reduce(function, self.all(), initial)
+    def reduce(self, func, initial=None, reverse=False):
+        '''Reduce `self.all()` using `func`'''
+        items = self.all()
 
-    def reduce_right(self, function, initial=None):
-        '''Call native `reduce` on reversed `self.all()`'''
-        return reduce(function, reversed(self.all()), initial)
+        if reverse:
+            items = reversed(items)
+
+        result = initial
+        for item in items:
+            result = func(result, item)
+        return result
+
+    def reduce_right(self, func, initial=None):
+        '''Reduce reversed `self.all()` using `func`'''
+        return self.reduce(func, initial=initial, reverse=True)
 
     def pluck(self, column):
         '''Pluck `column` attribute values from `self.all()` results and return as list'''
@@ -132,6 +149,7 @@ class Query(orm.Query):
         return self.limit(per_page).offset((page - 1) * per_page)
 
     def paginate(self, page=1, per_page=None, error_out=True):
+        '''Return Pagination instance using already defined query parameters.'''
         if error_out and page < 1:
             raise IndexError
 
@@ -152,14 +170,17 @@ class Query(orm.Query):
         return Pagination(self, page, per_page, total, items)
 
     def advanced_search(self, search_dict):
+        '''Apply advanced search filters given `search_dict`.'''
         filters = [m for m in [model.advanced_search(search_dict) for model in self.all_entities] if m is not None]
         return self.filter(and_(*filters)) if filters else self
 
     def simple_search(self, search_string):
+        '''Apply simple search filters given `search_string`.'''
         filters = [m for m in [model.simple_search(search_string) for model in self.all_entities] if m is not None]
         return self.filter(or_(*filters)) if filters else self
 
     def search(self, search_string=None, search_dict=None, limit=None, offset=None):
+        '''Perform combination of simple/advanced searching with optional limit/offset support.'''
         query = self
 
         if search_string is not None:
@@ -177,12 +198,17 @@ class Query(orm.Query):
         return query
 
 
+##
+# Pagination class and usage adapated from Flask-SQLAlchemy: https://github.com/mitsuhiko/flask-sqlalchemy
+##
+
+
 class Pagination(object):
     '''
-    Internal helper class returned by :meth:`BaseQuery.paginate`.  You
+    Internal helper class returned by `BaseQuery.paginate`.  You
     can also construct it from any other SQLAlchemy query object if you are
     working with other libraries.  Additionally it is possible to pass `None`
-    as query object in which case the :meth:`prev` and :meth:`next` will
+    as query object in which case the `prev` and `next` will
     no longer work.
     '''
 
@@ -210,19 +236,18 @@ class Pagination(object):
         self.has_next = self.page < self.pages
 
     def prev(self, error_out=False):
-        '''Returns a :class:`Pagination` object for the previous page.'''
+        '''Returns a `Pagination` object for the previous page.'''
         assert self.query is not None, 'a query object is required for this method to work'
         return self.query.paginate(self.page - 1, self.per_page, error_out)
 
     def next(self, error_out=False):
-        '''Returns a :class:`Pagination` object for the next page.'''
+        '''Returns a `Pagination` object for the next page.'''
         assert self.query is not None, 'a query object is required for this method to work'
         return self.query.paginate(self.page + 1, self.per_page, error_out)
 
 
 def get_load_options(*columns):
-    '''
-    Attempt to extract a sqlalchemy object from `columns[0]`
+    '''Helper method that attempts to extract a sqlalchemy object from `columns[0]`
     and return remaining columns to apply to a query load method.
     '''
     model_inspect = inspect(columns[0], raiseerr=False)
